/*
 * Updated 2007/01/21
 * Ver.1.00
 * charset = UTF-8
 * 
 * Mail : info@graviness.com
 * Home : http://www.graviness.com/
 * 
 * Ref.
 * 	* STATE:
 * 		http://msdn.microsoft.com/workshop/author/dhtml/reference/properties/readystate_2.asp
 * 	* XMLHttpRequest Field:
 * 		http://msdn.microsoft.com/library/default.asp?url=/library/en-us/xmlsdk/html/7924f6be-c035-411f-acd2-79de7a711b38.asp
 * 		http://www.openspc2.org/blog/archives/2006/02/ie7b2.html
 * 	* Other
 * 		http://www.itmedia.co.jp/enterprise/articles/0608/10/news019_3.html
 * 		http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html
 * 		http://developer.mozilla.org/ja/docs/AJAX:Getting_Started
 * 		http://sakaguch.com/HTTPlog.html
 */



//import graviness-sys-Common.js

/**
 * Ajaxラッパークラス．
 * 
 * XMLHttpRequestオブジェクトのブラウザ間の差異を埋め，
 * 最低限のメンバを提供します．
 * 
 * * XMLHttpRequestメンバ一覧
 * 	* 各ブラウザ共通
 * 		Method
 * 			open
 * 			send
 * 			abort
 * 			getAllResponseHeaders
 * 			getResponseHeader
 * 			setRequestHeader
 * 		Field
 * 			onreadystatechange
 * 			readyState
 * 			status (opera?)
 * 			responseText
 * 			responseXML
 * 			statusText (opera?)
 * 	* Internet Explorer 6
 * 		responseBody
 * 		responseStream
 * 	* Mozilla Firefox 2
 * 		channel = ??? #Permission denied to get property XMLHttpRequest.channel
 * 		overrideMimeType = function() {...}
 * 		multipart = false
 * 		onload = null
 * 		onerror = null
 * 		onprogress = null
 * 		addEventListener = function() {...}
 * 		removeEventListener = function() {...}
 * 		dispatchEvent = function() {...}
 * 		getInterface = function() {...} #Permission denied to call this method
 * 	* Opera 8
 * 		(None)
 * 	* Safari 2
 * 		overrideMimeType = function() {...}
 * 		onload = null
 * 
 * @see graviness-ajax-SimpleAjax
 */
function class__Ajaxor__(window) {
	var className = Object.getClassName(arguments.callee);

	var $kernel = Object.privateId();
	var $targetURI = Object.privateId();
	var $readyStateChangeListeners = Object.privateId();

	/**
	 * Ajaxorクラスのインスタンスを構築します．
	 */
	var F = window[className] = function() {
		// XMLHttpRequestオブジェクト．
		this[$kernel] = null;
		// 状態変更時リスナを要素に持つ配列．
		this[$readyStateChangeListeners] = new Array();
		// 送信先URI．
		this[$targetURI] = null;

		this.getKernel().onreadystatechange = function(that, ls) {
			return function() {
				for (var i = 0, n = ls.length; i < n; i++) {
					if (ls[i] !== null)
						ls[i](that.getReadyState(), that);
				}
			}
		}(this, this[$readyStateChangeListeners]);
	};

	/**
	 * XMLHttpRequestオブジェクトの状態：未初期化．
	 * openメソッドが呼ばれていない状態です．
	 */
	F.STATE_UNINITIALIZED = 0;

	/**
	 * XMLHttpRequestオブジェクトの状態：データロード中．
	 * sendメソッドが呼ばれていない状態です．
	 */
	F.STATE_LOADING = 1;

	/**
	 * XMLHttpRequestオブジェクトの状態：データロード完了．
	 * sendメソッドが呼ばれた状態です．
	 */
	F.STATE_LOADED = 2;

	/**
	 * XMLHttpRequestオブジェクトの状態：データ受信中．
	 * いくつかのデータを受信しました．
	 * responseXML/responseText等のデータはまだ完全ではありません．
	 */
	F.STATE_INTERACTIVE = 3;

	/**
	 * XMLHttpRequestオブジェクトの状態：データ受信完了．
	 * すべてのデータを受け取りました．
	 * responseXML/responseText等のデータの利用が可能です．
	 */
	F.STATE_COMPLETED = 4;

	var FP = Object.getPrototype(arguments.callee, window);

	/**
	 * XMLHttpRequestオブジェクトを取得します．
	 * 
	 * ブラウザ固有な操作を扱うときなどに使用します．
	 * 
	 * @return XMLHttpRequestオブジェクト．
	 * @throw NotAvailableXMLHttpRequestObjectException
	 * 	XMLHttpRequestオブジェクトが取得できないときスローされます．
	 */
	FP.getKernel = function() {
		if (this[$kernel] !== null)
			return this[$kernel];

		if (window.XMLHttpRequest !== void 0) {
			this[$kernel] = new window.XMLHttpRequest();
		}

		else if (window.ActiveXObject !== void 0) {
			try {
				this[$kernel] = new window.ActiveXObject("MXSML2.XMLHTTP");
			} catch (e) {
				try {
					this[$kernel] =
						new window.ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) {
				}
			}
		}

		if (this[$kernel] !== null) {
			return this[$kernel];
		} else {
			throw new String("NotAvailableXMLHttpRequestObjectException@"
				+ className + "#getKernel()");
		}
	};

	// NOTE:
	// GETのとき，クライアント側ではブラウザのキャッシュを
	// 参照することがあり，最新のデータを使用しないことがあります．
	// このとき，サーバ側で“Pragma:no-cache”や“Cache-Control:no-cache”
	// などのキャッシュを無効にするヘッダを付けると効果があります．

	/**
	 * 要求を出すサーバを設定し，初期化します．
	 * 
	 * @param method	サーバに要求するメソッド(HTTPの命令)を指定します．
	 * 	ex.) GET/HEAD/POST/PUT/DELETE/TRACE/OPTIONS/CONNECT/LINK/UNLINK
	 * 	NOTE: HTTPメソッドでPOSTするとき，
	 * 	以下の方法でMIMEタイプを変更する必要があります．(下記参照)
	 * 	#setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	 * 	Ref. <a href="http://developer.mozilla.org/ja/docs/AJAX:Getting_Started">AJAX:Getting Started@mozilla developer center</a>
	 * @param uri	送信先URI．
	 * 	セキュリティ上，異なるドメインを指定できません．
	 * @param async	真のとき非同期通信，偽のとき同期通信．
	 * 	真のとき，JavaScript関数の動作は，サーバから応答が返らなくても続けることができます．指定しないとき(デフォルト)は，非同期通信です．
	 * @throws uriに当該URIと異なるドメインを持つURIを指定したとき(セキュリティの関係)．
	 */
	FP.open = function(method, uri, async) {
		if (async === void 0)
			async = true;

		this.getKernel().open(method, this[$targetURI] = uri, Boolean(async));
	};

	/**
	 * Webサーバにデータを送信します．
	 * 
	 * @param data	Webサーバに送信するデータを指定します．
	 * 	送信するデータがないとき，#send()とします．
	 */
	FP.send = function(data) {
		if (data === void 0)
			data = null;

		this.getKernel().send(data);
	};

	/**
	 * HTTPリクエストを中止します．
	 * 
	 * NOTE: Mozilla Firefox2.0では無効のように見えます．
	 * @throws #getReadyState()で得られる値が，STATE_UNINITIALIZED(0), STATE_COMPLETEDのときコールする(？)．
	 */
	FP.abort = function() {
		this.getKernel().abort();
	};

	/**
	 * 送信先のターゲットとなるURIを取得します．
	 * 
	 * openメソッドで指定したURIをそのまま取得します．
	 * @return 送信先URI．
	 * 	openメソッド呼び出し前にコールしたとき，null．
	 */
	FP.getTargetURI = function() {
		return this[$targetURI];
	};

	/**
	 * Webサーバに送信するリクエストヘッダを設定します．
	 * 
	 * openメソッド呼出し後に有効です．
	 * @throws openメソッド呼び出し前にコールしたとき．
	 */
	FP.setRequestHeader = function(filedName, fieldValue) {
		this.getKernel().setRequestHeader(filedName, fieldValue);
	};

	/**
	 * Webサーバからのレスポンスヘッダの内容を取得します．
	 * 
	 * @param fieldName	ヘッダのフィールド名．
	 * 	ex.) "Date", "Server", "Content-Type", etc...
	 * @return	フィールド名に対応するフィールド値．
	 * 	ex.) "Sun, 30 Apr 2010 01:23:45 GMT", "Apache", "text/xml"
	 * 	フィールド名に対応するフィールド値がないとき，nullを返します．
	 * @throws データ受信前にコールしたとき．
	 */
	FP.getResponseHeader = function(fieldName) {
		var fieldValue = this.getKernel().getResponseHeader(fieldName);
		if (fieldValue === "")
			return null;
		return fieldValue;
	};

	/**
	 * Webサーバから受信したレスポンスヘッダの内容を全て取得します．
	 * 
	 * @return	ヘッダの内容を以下のフォーマットで返します(文字列)．
	 * 	フォーマット: "<var>fieldName1</var>: <var>fieldValue1</var>\n<var>fieldName2</var>: <var>fieldValue3</var>...\n"
	 * @throws データ受信前にコールしたとき．
	 */
	FP.getAllResponseHeaders = function() {
		return this.getKernel().getAllResponseHeaders();
	};

	/**
	 * このXMLHttpRequestオブジェクトの状態を取得します．
	 * 
	 * @return XMLHttpRequestオブジェクトの状態を示す整数値．
	 * @see STATE_UNINITIALIZED
	 * @see STATE_LOADING
	 * @see STATE_LOADED
	 * @see STATE_INTERACTIVE
	 * @see STATE_COMPLETED
	 */
	FP.getReadyState = function() {
		return this.getKernel().readyState;
	};

	/**
	 * Webサーバから受信したHTTPステータスコードを取得します．
	 * 
	 * このメソッドは，データ受信後(readyState == COMPLETED)に有効です．
	 * このメソッドが，200(OK)を返すとき，正常終了を意味します．
	 * Ref. <a href="http://ja.wikipedia.org/wiki/HTTP%E3%82%B9%E3%83%86%E3%83%BC%E3%82%BF%E3%82%B9%E3%82%B3%E3%83%BC%E3%83%89">HTTPステータスコード@wikipedia</a>
	 * @see #getStatusText()
	 * @return HTTPステータスコード．
	 * @throws 受信前に呼び出すとしたとき．
	 */
	FP.getStatus = function() {
		return this.getKernel().status;
	};

	/**
	 * Webサーバから受信したHTTPステータスコードに対応する文字列を取得します．
	 * 
	 * このメソッドは，データ受信後(readyState == COMPLETED)に有効です．
	 * このメソッドが，OK(200)を返すとき，正常終了を意味します．
	 * Ref. <a href="http://ja.wikipedia.org/wiki/HTTP%E3%82%B9%E3%83%86%E3%83%BC%E3%82%BF%E3%82%B9%E3%82%B3%E3%83%BC%E3%83%89">HTTPステータスコード@wikipedia</a>
	 * @see #getStatus()
	 * @return HTTPステータスコードに対応する文字列．
	 */
	FP.getStatusText = function() {
		try {
			return this.getKernel().statusText;
		} catch (e) {
			// IEでは，データ受信前に実行すると例外がスローされる．
			window.errorMessage(e);
			// 返り値はMozillaに統一．
			return "";
		}
	};

	/**
	 * 受信したデータをXMLオブジェクト形式を取得します．
	 * 
	 * @return 受信したデータ(XMLオブジェクト形式)．
	 */
	FP.getResponseXML = function() {
		try {
			return this.getKernel().responseXML;
		} catch (e) {
			// IEでは，データ受信前に実行すると例外がスローされる．
			window.errorMessage(e);
			// 返り値はMozilla側に統一．
			return null;
		}
	};

	/**
	 * 受信したデータをテキスト形式で取得します．
	 * 
	 * @return 受信したデータ(テキスト形式)．
	 */
	FP.getResponseText = function() {
		try {
			return this.getKernel().responseText;
		} catch (e) {
			// IEでは，データ受信前に実行すると例外がスローされる．
			window.errorMessage(e);
			// 返り値はMozilla側に統一．
			return "";
		}
	};

	/**
	 * XMLHttpRequestオブジェクトの状態変更時リスナ(関数)を登録します．
	 * 
	 * 登録されたリスナは，当該XMLHttpRequestオブジェクトの状態が変化するたびに
	 * 実行されます．
	 * 状態とは，XMLHttpRequestオブジェクトのreadyStateフィールドです．
	 * リスナとなる関数は次の形式を持ちます．
	 * 	function listener(int readyState, Ajaxor ajaxor)
	 * readyStateはそのときの状態を示す整数値で，ajaxorはこのインスタンスです．
	 * Ref. XMLHttpRequest#onreadystatechange
	 * 
	 * @param listener	リスナとなる関数．関数は次の形式を持ちます．
	 * 	function(int readyState, Ajaxor ajaxor)
	 * @throws IllegalArgumentsException	listenerが不正なとき．
	 */
	FP.addReadyStateChangeListener = function(listener) {
		if (!Boolean(listener) || typeof listener != "function")
			throw "IllegalArgumentsException@" + className + "#addReadyStateChangeListener(Function)";

		this[$readyStateChangeListeners].push(listener);
	};

	/**
	 * 登録した状態変更時リスナ(関数)を削除します．
	 * 
	 * @param listener	リスナ(関数)．
	 * @return 削除に成功したとき真．listenerが存在しないなど，失敗したとき偽．
	 * @throws IllegalArgumentsException	listenerが不正なとき．
	 */
	FP.removeReadyStateChangeListener = function(listener) {
		if (!Boolean(listener) || typeof listener != "function")
			throw "IllegalArgumentsException@" + className + "#addReadyStateChangeListener(Function)";

		var ls = this[$readyStateChangeListeners];
		for (var i = 0, n = ls.length; i < n; i++) {
			if (ls[i] === listener) {
				ls[i] = null;
				return true;
			}
		}

		return false;
	};

} class__Ajaxor__(window);
